/* Copyright (c) 1989 by Carrick Sean Casey. All rights reserved. */
/* For copying and distribution information, see the file "copyright.h." */

/* routines for reading the .forumrc initialization file */

#include "../copyright.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/dir.h>
#include "../forumnet.h"
#include "externs.h"

int startuperrors = 0;		/* set when startup file errors are found */
char rcpath[MAXNAMLEN+1];	/* path of startup file */

/* read and parse the startup file */

readforumrc()
{
	FILE *fp, *openforumrc();
	char s[133];

	/* open the file */
	if ((fp = openforumrc()) == 0)
		return(0);

	/* read it in and parse */
	while(fgets(s, 133, fp) != NULL)
		doforumrcline(s);
	fclose(fp);

	/* clean up the output a bit */
	if (startuperrors)
		putchar('\n');
}


/* find the filename and open it - return a pointer to the */
/* open file or zero on failure */

FILE *
openforumrc()
{
	int mustopen = 0;
	FILE *fp;
	char *rcfile, *homedir, *getenv();

	/* get name of file to use */
	if ((rcfile = getenv("FORUMRC")) != NULL) {

		/* user specified a specific file */
		mustopen++;

		if (*rcfile == '/') {
			/* absolute pathname */
			strcpy(rcpath, rcfile);
		} else {
			/* relative pathname to home */
			if ((homedir = getenv("HOME")) == NULL) {
				fprintf(stderr, "HOME not set.\n");
				return(0);
			}
			sprintf(rcpath, "%s/%s", homedir, rcfile);
		}
	} else {
		if ((homedir = getenv("HOME")) == NULL) {
			fprintf(stderr, "HOME not set.\n");
			return(0);
		}
		sprintf(rcpath, "%s/.%src", homedir, PROGNAME);
	}

	/* open the file */
	if ((fp = fopen(rcpath, "r")) == NULL) {
		if (mustopen) {
			/* since we specified a file, give an error */
			fprintf(stderr, "Couldn't open \"%s\": ", rcpath);
			perror("");
			return(0);
		} else
			/* no file specified */
			return(0);
	}
	return(fp);
}


/* parse one line of the file - the line usually has a newline appended */

doforumrcline(s)
char *s;
{
	char *p, *findspace();

	/* work through leading whitespace */
	while (*s == ' ' || *s == '\t')
		s++;
		
	/* ignore comment or blank lines */
	if (*s == '#' || *s == '\n' || *s == '\0')
		return(0);

	/* figger out the keyword */
	else if (strncmp(s, "cutetime", 8) == 0) {
		m_watchtime = 0;
		return(0);
	} else if (strncmp(s, "cmdchar", 7) == 0) {
		if ((p = findspace(s)) != 0) {
			cmdchar = *p;
			return(0);
		}
	} else if (strncmp(s, "nowhoheader", 11) == 0) {
		m_whoheader = 0;
		return(0);
	} else if (strncmp(s, "nogroupheader", 12) == 0) {
		m_groupheader = 1;
		return(0);
	} else if (strncmp(s, "nomesg", 6) == 0) {
		m_nomesg = 1;
		return(0);
	} else if (strncmp(s, "watchtime", 9) == 0) {
		m_watchtime = 1;
		return(0);
	} else {
		if (!startuperrors) {
			fprintf(stderr, "I don't understand the following ");
			fprintf(stderr, "lines in \"%s\":\n", rcpath);
			startuperrors++;
		}
		fprintf(stderr, "-> %s", s);
		return(0);
	}
}
