/* Copyright (c) 1989 by Carrick Sean Casey. All rights reserved. */
/* For copying and distribution information, see the file "copyright.h." */

/* get a line from the user, with input processing */
/* returns chars in line, or 0 if user erased to beginning of line */

#include "../copyright.h"
#include <stdio.h>
#include <ctype.h>
#include "../forumnet.h"
#include "externs.h"

getline(s, prompt, expand)
char *s;
char *prompt;
int expand;
{
	int c;
	char cc;
	char *p = s;
	int count = 0;
	char *lastmessage(), *namecomget();

	if (prompt) {
		fputs(prompt, stdout);
		fflush(stdout);
	}
		
	for(;;) {
		c = getchar();

		/* reset if we were restarted */
		if (continued) {
			continued = 0;
			p = s;
			count = 0;
		}

		/* regular ascii */
		if (c > '\037' && c < '\177') {
			if (count == (MAX_INPUTSTR - 1)) {
				write(1, "\007", 1);
				continue;
			}
			*p++ = (char)c;
			count++;
			cc = c;
			write(1, &cc, 1);
		} else
			switch(c) {

#ifdef NEVERDEFFED
			case '\033':	/* ESC - name completion */
			      {
				int tcount = count;
				char *tp = p;
				char *wp;

				*p = '\0';
				/* back to beginning of word */
				while (tcount && !isspace(*(tp-1)))
					tp--, tcount--;

				/* throw expansion on it */
				if ((wp = namecomget(tp)) != 0) {
					sprintf(p, "%s ", wp);
					fputs(p, stdout);
					count = count + strlen(p);
					p = s + count;
					fflush(stdout);
				}
			      }
				break;
#endif NEVERDEFFED

			case '\t':	/* TAB - enter personal message line */
				if (!expand || histcount() == 0)
					break;
				if (!count) {
					sprintf(s, "/m %s ", histget());
					count = strlen(s);
					p = s + count;
					fputs(s, stdout);
					fflush(stdout);
				} else {
					while (count && isspace(*(p-1))) {
						p--, count--;
						fputs("\b \b", stdout);
					}
					while (count && !isspace(*(p-1))) {
						p--, count--;
						fputs("\b \b", stdout);
					}
					sprintf(p, "%s ", histget());
					fputs(p, stdout);
					count = count + strlen(p);
					p = s + count;
					fflush(stdout);
				}
				break;

			case '\022':	/* ^R  - redraw line */
				if (count) {
					*p = '\0';
					fputs("\r\n", stdout);
					if (prompt)
						fputs(prompt, stdout);
					fputs(s, stdout);
					fflush(stdout);
				}
				break;
				
			case '\025':	/* ^U  - kill line */
				while (count) {
					fputs("\b \b", stdout);
					count--;
				}
				fflush(stdout);
				*s = '\0';
				if (!prompt)
					return(0);
				break;

			case '\027':	/* ^W - delete word */
				while (count && isspace(*(p-1))) {
					p--, count--;
					fputs("\b \b", stdout);
				}
				while (count && !isspace(*(p-1))) {
					p--, count--;
					fputs("\b \b", stdout);
				}
				fflush(stdout);
				if (!count) {
					*s = '\0';
					if (!prompt)
						return(0);
				}
				break;

			case '\177':	/* delete or... */
			case '\b':	/* ^H - backspace one character */
				if (count) {
					fputs("\b \b", stdout);
					p--, count--;
				}
				if (!count) {
					*s = '\0';
					if (!prompt)
						return(0);
				}
				fflush(stdout);
				break;
				
			case '\n':	/* new line - enter the next */
				if (!count)
					*p = '\n';
				else 
					*p = '\0';
				putchar('\n');
				return(count);
				break;

			default:
				/* control characters */
				break;
			}
	}
}
